package oclstdlibplugin.model;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jdt.annotation.NonNull;
import org.eclipse.ocl.examples.domain.elements.DomainOperation;
import org.eclipse.ocl.examples.domain.utilities.DomainUtil;
import org.eclipse.ocl.examples.pivot.AnyType;
import org.eclipse.ocl.examples.pivot.ExpressionInOCL;
import org.eclipse.ocl.examples.pivot.PrimitiveType;
import org.eclipse.ocl.examples.pivot.Query;
import org.eclipse.ocl.examples.pivot.context.ModelContext;
import org.eclipse.ocl.examples.pivot.delegate.OCLDelegateDomain;
import org.eclipse.ocl.examples.pivot.helper.OCLHelper;
import org.eclipse.ocl.examples.pivot.manager.MetaModelManager;
import org.eclipse.ocl.examples.pivot.utilities.PivotEnvironmentFactory;
import org.eclipse.ocl.examples.xtext.base.utilities.BaseCSResource;
import org.eclipse.ocl.examples.xtext.base.utilities.CS2PivotResourceAdapter;

import com.google.common.collect.Iterables;

public class OCLStdLibTest {
	MetaModelManager metaModelManager = new MetaModelManager();
	
	public OCLStdLibTest() {
		String testFile =
				"import 'http://www.eclipse.org/ocl/3.1.0/OCL.oclstdlib';\n"+
				"library lib : lib = 'http://www.eclipse.org/ocl/3.1.0/OCL.oclstdlib'{\n"+
				"    type Boolean : PrimitiveType conformsTo OclAny {\n"+
				"    	operation a(elem : Boolean) : Boolean {\n"+
				"           post a: result = elem;\n"+
				"       }\n"+
				"    }\n"+
				"    type Integer : PrimitiveType conformsTo Real {\n"+
				"    	operation test() : Integer =>'ics.eaat.abstractmodeller.model.TestNumericOperation';"+
				"    }\n"+				
				
				"}\n";		
			Resource pivotResource;
			try {
				pivotResource = doLoadFromString("string.oclstdlib", testFile);
				MetaModelManager metaModelManager = MetaModelManager.getAdapter(pivotResource.getResourceSet());
				PrimitiveType oclAnyType = metaModelManager.getBooleanType();
				Iterable<? extends DomainOperation> ownedOperations = metaModelManager.getAllOperations(oclAnyType, false);
				System.out.println("methods = " + Iterables.size(ownedOperations));
				org.eclipse.ocl.examples.pivot.OCL ocl = org.eclipse.ocl.examples.pivot.OCL.newInstance(new PivotEnvironmentFactory(pivotResource.getResourceSet().getPackageRegistry(), metaModelManager), pivotResource);
				OCLHelper helper = ocl.createOCLHelper();				   
				ExpressionInOCL exp = helper.createQuery("let x : Integer = 8 in x.test()");
				Query q = ocl.createQuery(exp);
				System.out.println("result = " + q.evaluate());
				
				metaModelManager.dispose();  	
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
	}
	
	/*public static void main(String args[]) {
		new OCLStdLibTest();
	}*/
	
	protected Resource doLoadFromString(@NonNull String fileName, @NonNull String testFile) throws Exception {
		URI libraryURI = getProjectFileURI(fileName);
		ModelContext modelContext = new ModelContext(metaModelManager, libraryURI);
		BaseCSResource xtextResource = (BaseCSResource) modelContext.createBaseResource(testFile);
		//assertNoResourceErrors("Load failed", xtextResource);
		CS2PivotResourceAdapter adapter = CS2PivotResourceAdapter.getAdapter(xtextResource, null);
		Resource pivotResource = adapter.getPivotResource(xtextResource);
		//assertNoResourceErrors("File Model", pivotResource);
		//assertNoUnresolvedProxies("File Model", pivotResource);
		//assertNoValidationErrors("File Model", pivotResource);
		return pivotResource;
	}
	
	protected @NonNull URI getProjectFileURI(String referenceName) {
		File projectFile = getProjectFile();
		return DomainUtil.nonNullState(URI.createFileURI(projectFile.toString() + "/" + referenceName));
	}	
	
	protected @NonNull String getProjectName() {
		return getClass().getPackage().getName().replace('.', '/') ;
	}
	
	protected @NonNull File getProjectFile() {
		String projectName = getProjectName();
		URL projectURL = getTestResource(projectName);	
		//assertNotNull(projectURL);
		return new File(projectURL.getFile());
	}	
	
	protected @NonNull URL getTestResource(@NonNull String resourceName) {
		URL projectURL = getClass().getClassLoader().getResource(resourceName);
		try {
			if ((projectURL != null) && Platform.isRunning()) {
				try {
					projectURL = FileLocator.resolve(projectURL);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		catch (Throwable e) {}
		return DomainUtil.nonNullState(projectURL);
	}	
}
